# ias (interaction score) 
# computes set-wise co-expression 
# x,y are different gene set expression matrices. 
# row-> gene, column-> sample
# Samples should be identical between expression matices


ias<-function(x,y,method='pearson'){
      
         if(ncol(x)!=ncol(y)) stop('The sample numbers of two expression profiles are different!')

         rx<-RRE.matrix(x)
         ry<-RRE.matrix(y)

         rx<-rx[upper.tri(rx)]
         ry<-ry[upper.tri(ry)]
	
         if(length(which(!is.finite(rx)))>0 || length(which(!is.finite(ry)))>0) return(c(NA,NA))
	 if(sd(rx)==0 || sd(ry)==0) return(c(NA,NA))

	 tmp<-cor.test(rx,ry,method=method)
	 if(is.na(tmp[[4]])) return(c(NA,NA))
         c(tmp[[4]],tmp[[3]])
}

#  implementation of rule-of-thumb band width selection by Scott (92)   

md.bw.estimation<-function(x){
         sigma<-apply(x,MARGIN=1,FUN=sd)
         h<-sigma*ncol(x)^(-1/(nrow(x)+4))
         h
}


#  multivariate product kernel     
#  x is an instance, y is a total set of samples, bw is a list of bandwidth 
#  using gaussian kernel
#  row of y means gene, column of y means sample 

md.kernel<-function(x,y){
          kk<-c()
          h<-md.bw.estimation(y)
      
          for(i in 1:ncol(y)){
             k<-c()
             for(j in 1:length(x)){
	         k<-c(k,(2*pi)^(-0.5)*exp(-(x[j]-y[j,i])^2/(2*(h[j]^2))))
	         k<-prod(k)
	     }
	     kk<-c(kk,k)
          }
          1/(ncol(y)*prod(h))*sum(kk)
}



# kernel density estimation of a sample set

md.kernel.set<- function(x){
           re<-c()
	   for(i in 1:ncol(x)) re<-c(md.kernel(x[,i],x[,-i]),re)
           re
}
 	    

# Renyi's quadratic relative entropy
# Sample-wise entropy matrix 
    
RRE.matrix<-function(x){
            k<-ncol(x)
            re<-matrix(nrow=k,ncol=k)
	    tm<-md.kernel.set(x)                
	    for(i in 1:k){
	       for(j in 1:k){re[i,j]<-log((tm[i]+.Machine$double.eps)/(tm[j]+.Machine$double.eps))}
	    }
	    re
}

